@@ -70,24 +70,37 @@ bool FILE_XFER_BACKOFF::ok_to_transfer() {
     return (dt <= 0);
 }
 
+// A transfer has failed.
+// Back off transfers (project-wide) if needed.
+//
 void FILE_XFER_BACKOFF::file_xfer_failed(PROJECT* p) {
+    // If we're already backed off, ignore this failure.
+    // If we start several transfers at once
+    // (say, N output files of a job) and they all fail,
+    // we don't want to back off N times, which could be hours.
+    //
+    if (gstate.now < next_xfer_time) {
+        return;
+    }
+
     file_xfer_failures++;
     if (file_xfer_failures < FILE_XFER_FAILURE_LIMIT) {
         next_xfer_time = 0;
-    } else {
-        double backoff = calculate_exponential_backoff(
-            file_xfer_failures,
-            gstate.pers_retry_delay_min,
-            gstate.pers_retry_delay_max
-        );
-        if (log_flags.file_xfer_debug) {
-            msg_printf(p, MSG_INFO,
-                "[file_xfer] project-wide xfer delay for %f sec",
-                backoff
-            );
-        }
-        next_xfer_time = gstate.now + backoff;
+        return;
     }
+    double backoff = calculate_exponential_backoff(
+        file_xfer_failures,
+        gstate.pers_retry_delay_min,
+        gstate.pers_retry_delay_max
+    );
+    if (log_flags.file_xfer_debug) {
+        msg_printf(p, MSG_INFO,
+            "[file_xfer] project-wide %s delay for %f sec",
+            is_upload?"upload":"download",
+            backoff
+        );
+    }
+    next_xfer_time = gstate.now + backoff;
 }
 
 void FILE_XFER_BACKOFF::file_xfer_succeeded() {
@@ -389,13 +402,13 @@ int FILE_INFO::parse(XML_PARSER& xp) {
             retval = pfxp->parse(xp);
 #ifdef SIM
             delete pfxp;
-            continue;
-#endif
+#else
             if (!retval) {
                 pers_file_xfer = pfxp;
             } else {
                 delete pfxp;
             }
+#endif
             continue;
         }
         if (xp.match_tag("file_xfer")) {
@@ -672,7 +685,7 @@ bool FILE_INFO::had_failure(int& failnum) {
 
 void FILE_INFO::failure_message(string& s) {
     char buf[1024];
-    snprintf(buf, sizeof(buf), 
+    snprintf(buf, sizeof(buf),
         "<file_xfer_error>\n"
         "  <file_name>%s</file_name>\n"
         "  <error_code>%d (%s)</error_code>\n",
@@ -1336,3 +1349,92 @@ double RUN_MODE::delay() {
         return 0;
     }
 }
+
+int USER_CPID::parse(XML_PARSER& xp) {
+    clear();
+    while (!xp.get_tag()) {
+        if (xp.match_tag("/user_cpid")) {
+            if (!strlen(email_hash) || !strlen(cpid) || !time) {
+                msg_printf(0, MSG_INTERNAL_ERROR, "USER_CPID::parse() failed");
+                return ERR_XML_PARSE;
+            }
+            break;
+        }
+        if (xp.parse_str("email_hash", email_hash, sizeof(email_hash))) continue;
+        if (xp.parse_str("cpid", cpid, sizeof(cpid))) continue;
+        if (xp.parse_double("time", time)) continue;
+    }
+    return 0;
+}
+
+int USER_CPID::write(MIOFILE& out) {
+    out.printf(
+        "   <user_cpid>\n"
+        "      <email_hash>%s</email_hash>\n"
+        "      <cpid>%s</cpid>\n"
+        "      <time>%f</time>\n"
+        "   </user_cpid>\n",
+        email_hash, cpid, time
+    );
+    return 0;
+}
+
+int USER_CPIDS::parse(XML_PARSER& xp) {
+    while (!xp.get_tag()) {
+        if (xp.match_tag("/user_cpids")) {
+            break;
+        }
+        if (xp.match_tag("user_cpid")) {
+            USER_CPID uc;
+            int retval = uc.parse(xp);
+            if (retval) continue;
+            cpids.push_back(uc);
+        }
+    }
+    return 0;
+}
+
+int USER_CPIDS::write(MIOFILE& out) {
+    out.printf("<user_cpids>\n");
+    for (USER_CPID &cpid: cpids) {
+        cpid.write(out);
+    }
+    out.printf("</user_cpids>\n");
+    return 0;
+}
+
+USER_CPID* USER_CPIDS::lookup(const char* email_hash) {
+    for (USER_CPID &cpid: cpids) {
+        if (!strcmp(cpid.email_hash, email_hash)) {
+            return &cpid;
+        }
+    }
+    return NULL;
+}
+
+// if empty, initialize from projects
+// (should get done once, when updating to this client version)
+//
+void USER_CPIDS::init_from_projects() {
+    for (PROJECT *p: gstate.projects) {
+        if (!strlen(p->email_hash) || !strlen(p->cross_project_id) || !p->user_create_time) {
+            continue;
+        }
+        USER_CPID* ucp = lookup(p->email_hash);
+        if (ucp) {
+            if (p->user_create_time < ucp->time) {
+                strcpy(ucp->cpid, p->cross_project_id);
+                ucp->time = p->user_create_time;
+            }
+        } else {
+            USER_CPID uc;
+            strcpy(uc.email_hash, p->email_hash);
+            strcpy(uc.cpid, p->cross_project_id);
+            uc.time = p->user_create_time;
+            cpids.push_back(uc);
+
+        }
+    }
+}
+
+USER_CPIDS user_cpids;
